@@ -32,7 +32,7 @@ module Network.HTTP.Client.Request
 
 import Data.Int (Int64)
 import Data.Maybe (fromMaybe, isJust, isNothing)
-import Data.Monoid (mempty, mappend)
+import Data.Monoid (mempty, mappend, (<>))
 import Data.String (IsString(..))
 import Data.Char (toLower)
 import Control.Applicative as A ((<$>))
@@ -103,13 +103,15 @@ parseUrlThrow =
 -- space, e.g.:
 --
 -- @@@
--- parseRequeset "POST http://httpbin.org/post"
+-- parseRequest "POST http://httpbin.org/post"
 -- @@@
 --
 -- Note that the request method must be provided as all capital letters.
 --
 -- 'Request' created by this function won't cause exceptions on non-2XX
--- response status codes.
+-- response status codes. 
+--
+-- To create a request which throws on non-2XX status codes, see 'parseUrlThrow'
 --
 -- @since 0.4.30
 parseRequest :: MonadThrow m => String -> m Request
@@ -138,17 +140,7 @@ parseRequest_ = either throw id . parseRequest
 -- | Add a 'URI' to the request. If it is absolute (includes a host name), add
 -- it as per 'setUri'; if it is relative, merge it with the existing request.
 setUriRelative :: MonadThrow m => Request -> URI -> m Request
-setUriRelative req uri =
-#ifndef MIN_VERSION_network
-#define MIN_VERSION_network(x,y,z) 1
-#endif
-#if MIN_VERSION_network(2,4,0)
-    setUri req $ uri `relativeTo` getUri req
-#else
-    case uri `relativeTo` getUri req of
-        Just uri' -> setUri req uri'
-        Nothing   -> throwM $ InvalidUrlException (show uri) "Invalid URL"
-#endif
+setUriRelative req uri = setUri req $ uri `relativeTo` getUri req
 
 -- | Extract a 'URI' from the request.
 --
