@@ -7,8 +7,11 @@ module Network.HTTP.Client.Types
     ( BodyReader
     , Connection (..)
     , StatusHeaders (..)
-    , ConnectionClosed (..)
     , HttpException (..)
+    , HttpExceptionContent (..)
+    , unHttpExceptionContentWrapper
+    , throwHttp
+    , toHttpException
     , Cookie (..)
     , CookieJar (..)
     , Proxy (..)
@@ -31,17 +34,17 @@ module Network.HTTP.Client.Types
     , ConnKey (..)
     , ProxyOverride (..)
     , StreamFileStatus (..)
+    , ResponseTimeout (..)
     ) where
 
 import qualified Data.Typeable as T (Typeable)
 import Network.HTTP.Types
-import Control.Exception (Exception, IOException, SomeException)
+import Control.Exception (Exception, SomeException, throwIO)
 import Data.Word (Word64)
 import qualified Data.ByteString as S
 import qualified Data.ByteString.Lazy as L
 import Blaze.ByteString.Builder (Builder, fromLazyByteString, fromByteString, toLazyByteString)
 import Data.Int (Int64)
-import Data.Default.Class
 import Data.Foldable (Foldable)
 import Data.Monoid
 import Data.String (IsString, fromString)
@@ -85,40 +88,115 @@ data Connection = Connection
 data StatusHeaders = StatusHeaders Status HttpVersion RequestHeaders
     deriving (Show, Eq, Ord, T.Typeable)
 
-data ConnectionClosed = ConnectionClosed
-  deriving (Eq, Show, T.Typeable)
+-- | A newtype wrapper which is not exported from this library but is an
+-- instance of @Exception@. This allows @HttpExceptionContent@ to be thrown
+-- (via this wrapper), but users of the library can't accidentally try to catch
+-- it (when they /should/ be trying to catch 'HttpException').
+--
+-- @since 0.5.0
+newtype HttpExceptionContentWrapper = HttpExceptionContentWrapper
+    { unHttpExceptionContentWrapper :: HttpExceptionContent
+    }
+    deriving (Show, T.Typeable)
+instance Exception HttpExceptionContentWrapper
 
-instance Exception ConnectionClosed
+throwHttp :: HttpExceptionContent -> IO a
+throwHttp = throwIO . HttpExceptionContentWrapper
 
-data HttpException = StatusCodeException Status ResponseHeaders CookieJar
-                   | InvalidUrlException String String
-                   | TooManyRedirects [Response L.ByteString]  -- ^ List of encountered responses containing redirects in reverse chronological order; including last redirect, which triggered the exception and was not followed.
-                   | UnparseableRedirect (Response L.ByteString) -- ^ Response containing unparseable redirect.
-                   | TooManyRetries
-                   | HttpParserException String
-                   | HandshakeFailed
+toHttpException :: Request -> HttpExceptionContentWrapper -> HttpException
+toHttpException req (HttpExceptionContentWrapper e) = HttpExceptionRequest req e
+
+-- | An exception which may be generated by this library
+--
+-- @since 0.5.0
+data HttpException
+    = HttpExceptionRequest Request HttpExceptionContent
+    -- ^ Most exceptions are specific to a 'Request'. Inspect the
+    -- 'HttpExceptionContent' value for details on what occurred.
+    --
+    -- @since 0.5.0
+    | InvalidUrlException String String
+    -- ^ A URL (first field) is invalid for a given reason
+    -- (second argument).
+    --
+    -- @since 0.5.0
+    deriving (Show, T.Typeable)
+instance Exception HttpException
+
+data HttpExceptionContent
+                   = StatusCodeException (Response ()) S.ByteString
+                   -- ^ Generated by the @parseUrlThrow@ function when the
+                   -- server returns a non-2XX response status code.
+                   --
+                   -- May include the beginning of the response body.
+                   --
+                   -- @since 0.5.0
+                   | TooManyRedirects [Response L.ByteString]
+                   -- ^ The server responded with too many redirects for a
+                   -- request.
+                   --
+                   -- Contains the list of encountered responses containing
+                   -- redirects in reverse chronological order; including last
+                   -- redirect, which triggered the exception and was not
+                   -- followed.
+                   --
+                   -- @since 0.5.0
                    | OverlongHeaders
+                   -- ^ Either too many headers, or too many total bytes in a
+                   -- single header, were returned by the server, and the
+                   -- memory exhaustion protection in this library has kicked
+                   -- in.
+                   --
+                   -- @since 0.5.0
                    | ResponseTimeout
-                   | FailedConnectionException String Int
-                   -- ^ host/port
+                   -- ^ The server took too long to return a response. This can
+                   -- be altered via 'responseTimeout' or
+                   -- 'managerResponseTimeout'.
                    --
-                   -- Note that in old versions of http-client and
-                   -- http-conduit, this exception would indicate a failed
-                   -- attempt to create a connection. However, since (at least)
-                   -- http-client 0.4, it indicates a timeout occurred while
-                   -- trying to establish the connection. For more information
-                   -- on this, see:
+                   -- @since 0.5.0
+                   | ConnectionTimeout
+                   -- ^ Attempting to connect to the server timed out.
                    --
-                   -- <https://github.com/snoyberg/http-client/commit/b86b1cdd91e56ee33150433dedb32954d2082621#commitcomment-10718689>
-                   | FailedConnectionException2 String Int Bool SomeException -- ^ host\/port\/secure
-                   | ExpectedBlankAfter100Continue
+                   -- @since 0.5.0
+                   | ConnectionFailure SomeException
+                   -- ^ An exception occured when trying to connect to the
+                   -- server.
+                   --
+                   -- @since 0.5.0
                    | InvalidStatusLine S.ByteString
+                   -- ^ The status line returned by the server could not be parsed.
+                   --
+                   -- @since 0.5.0
                    | InvalidHeader S.ByteString
-                   | InternalIOException IOException
-                   | ProxyConnectException S.ByteString Int (Either S.ByteString HttpException) -- ^ host/port
+                   -- ^ The given response header line could not be parsed
+                   --
+                   -- @since 0.5.0
+                   | InternalException SomeException
+                   -- ^ An exception was raised by an underlying library when
+                   -- performing the request. Most often, this is caused by a
+                   -- failing socket action or a TLS exception.
+                   --
+                   -- @since 0.5.0
+                   | ProxyConnectException S.ByteString Int Status
+                   -- ^ A non-200 status code was returned when trying to
+                   -- connect to the proxy server on the given host and port.
+                   --
+                   -- @since 0.5.0
                    | NoResponseDataReceived
-                   | TlsException SomeException
+                   -- ^ No response data was received from the server at all.
+                   -- This exception may deserve special handling within the
+                   -- library, since it may indicate that a pipelining has been
+                   -- used, and a connection thought to be open was in fact
+                   -- closed.
+                   --
+                   -- @since 0.5.0
                    | TlsNotSupported
+                   -- ^ Exception thrown when using a @Manager@ which does not
+                   -- have support for secure connections. Typically, you will
+                   -- want to use @tlsManagerSettings@ from @http-client-tls@
+                   -- to overcome this.
+                   --
+                   -- @since 0.5.0
                    | WrongRequestBodyStreamSize Word64 Word64
                    -- ^ The request body provided did not match the expected size.
                    --
@@ -126,34 +204,43 @@ data HttpException = StatusCodeException Status ResponseHeaders CookieJar
                    --
                    -- @since 0.4.31
                    | ResponseBodyTooShort Word64 Word64
-                   -- ^ Expected size/actual size.
+                   -- ^ The returned response body is too short. Provides the
+                   -- expected size and actual size.
                    --
-                   -- Since 1.9.4
+                   -- @since 0.5.0
                    | InvalidChunkHeaders
-                   -- ^
+                   -- ^ A chunked response body had invalid headers.
                    --
-                   -- Since 1.9.4
+                   -- @since 0.5.0
                    | IncompleteHeaders
+                   -- ^ An incomplete set of response headers were returned.
+                   --
+                   -- @since 0.5.0
                    | InvalidDestinationHost S.ByteString
+                   -- ^ The host we tried to connect to is invalid (e.g., an
+                   -- empty string).
                    | HttpZlibException ZlibException
-                   -- ^
+                   -- ^ An exception was thrown when inflating a response body.
                    --
-                   -- Since 0.3
+                   -- @since 0.5.0
                    | InvalidProxyEnvironmentVariable Text Text
-                   -- ^ Environment name and value
+                   -- ^ Values in the proxy environment variable were invalid.
+                   -- Provides the environment variable name and its value.
                    --
-                   -- Since 0.4.7
-                   | ResponseLengthAndChunkingBothUsed
-                   -- ^ Detect a case where both the @content-length@ header
-                   -- and @transfer-encoding: chunked@ are used. Since 0.4.8.
+                   -- @since 0.5.0
+                   | ConnectionClosed
+                   -- ^ Attempted to use a 'Connection' which was already closed
                    --
-                   -- Since 0.4.11 this exception isn't thrown anymore.
-                   | TlsExceptionHostPort SomeException S.ByteString Int
-                   -- ^ TLS exception, together with the host and port
-                   --
-                   -- @since 0.4.24
+                   -- @since 0.5.0
+                   | InvalidProxySettings Text
+                   -- ^ Proxy settings are not valid (Windows specific currently)
+                   -- @since 0.5.7
     deriving (Show, T.Typeable)
-instance Exception HttpException
+
+-- Purposely not providing this instance, since we don't want users to
+-- accidentally try and catch these exceptions instead of HttpException
+--
+-- instance Exception HttpExceptionContent
 
 
 -- This corresponds to the description of a cookie detailed in Section 5.3 \"Storage Model\"
@@ -189,15 +276,12 @@ instance Ord Cookie where
     | cookie_creation_time c1 > cookie_creation_time c2 = GT
     | otherwise = LT
 
-instance Default CookieJar where
-  def = CJ []
-
 instance Eq CookieJar where
   (==) cj1 cj2 = (DL.sort $ expose cj1) == (DL.sort $ expose cj2)
 
 -- | Since 1.9
-instance Monoid CookieJar where
-  mempty = def
+instance Data.Monoid.Monoid CookieJar where
+  mempty = CJ []
   (CJ a) `mappend` (CJ b) = CJ (DL.nub $ DL.sortBy compare' $ a `mappend` b)
     where compare' c1 c2 =
             -- inverse so that recent cookies are kept by nub over older
@@ -277,6 +361,7 @@ simplify (RequestBodyBS bs) = Left (fromIntegral $ S.length bs, fromByteString b
 simplify (RequestBodyBuilder len b) = Left (len, b)
 simplify (RequestBodyStream i gp) = Right (Just i, gp)
 simplify (RequestBodyStreamChunked gp) = Right (Nothing, gp)
+simplify (RequestBodyIO _mbody) = error "FIXME No support for Monoid on RequestBodyIO"
 
 builderToStream :: (Int64, Builder) -> (Maybe Int64, GivesPopper ())
 builderToStream (len, builder) =
@@ -412,30 +497,21 @@ data Request = Request
     -- no redirects. Default value: 10.
     --
     -- Since 0.1.0
-    , checkStatus :: Status -> ResponseHeaders -> CookieJar -> Maybe SomeException
-    -- ^ Check the status code. Note that this will run after all redirects are
-    -- performed. Default: return a @StatusCodeException@ on non-2XX responses.
+    , checkResponse :: Request -> Response BodyReader -> IO ()
+    -- ^ Check the response immediately after receiving the status and headers.
+    -- This can be useful for throwing exceptions on non-success status codes.
     --
-    -- Since 0.1.0
-    , responseTimeout :: Maybe Int
+    -- In previous versions of http-client, this went under the name
+    -- @checkStatus@, but was renamed to avoid confusion about the new default
+    -- behavior (doing nothing).
+    --
+    -- @since 0.5.0
+    , responseTimeout :: ResponseTimeout
     -- ^ Number of microseconds to wait for a response. If
     -- @Nothing@, will wait indefinitely. Default: use
     -- 'managerResponseTimeout' (which by default is 30 seconds).
     --
     -- Since 0.1.0
-    , getConnectionWrapper :: Maybe Int
-                           -> HttpException
-                           -> IO (ConnRelease, Connection, ManagedConn)
-                           -> IO (Maybe Int, (ConnRelease, Connection, ManagedConn))
-    -- ^ Wraps the calls for getting new connections. This can be useful for
-    -- instituting some kind of timeouts. The first argument is the value of
-    -- @responseTimeout@. Second argument is the exception to be thrown on
-    -- failure.
-    --
-    -- Default: If @responseTimeout@ is @Nothing@, does nothing. Otherwise,
-    -- institutes timeout, and returns remaining time for @responseTimeout@.
-    --
-    -- Since 0.1.0
     , cookieJar :: Maybe CookieJar
     -- ^ A user-defined cookie jar.
     -- If 'Nothing', no cookie handling will take place, \"Cookie\" headers
@@ -467,6 +543,34 @@ data Request = Request
     }
     deriving T.Typeable
 
+-- | How to deal with timing out a response
+--
+-- @since 0.5.0
+data ResponseTimeout
+    = ResponseTimeoutMicro !Int
+    | ResponseTimeoutNone
+    | ResponseTimeoutDefault
+    deriving (Eq, Show)
+
+instance Show Request where
+    show x = unlines
+        [ "Request {"
+        , "  host                 = " ++ show (host x)
+        , "  port                 = " ++ show (port x)
+        , "  secure               = " ++ show (secure x)
+        , "  requestHeaders       = " ++ show (requestHeaders x)
+        , "  path                 = " ++ show (path x)
+        , "  queryString          = " ++ show (queryString x)
+        --, "  requestBody          = " ++ show (requestBody x)
+        , "  method               = " ++ show (method x)
+        , "  proxy                = " ++ show (proxy x)
+        , "  rawBody              = " ++ show (rawBody x)
+        , "  redirectCount        = " ++ show (redirectCount x)
+        , "  responseTimeout      = " ++ show (responseTimeout x)
+        , "  requestVersion       = " ++ show (requestVersion x)
+        , "}"
+        ]
+
 data ConnReuse = Reuse | DontReuse
     deriving T.Typeable
 
@@ -507,7 +611,7 @@ data Response body = Response
     --
     -- Since 0.1.0
     }
-    deriving (Show, Eq, T.Typeable, Functor, Foldable, Traversable)
+    deriving (Show, Eq, T.Typeable, Functor, Data.Foldable.Foldable, Data.Traversable.Traversable)
 
 newtype ResponseClose = ResponseClose { runResponseClose :: IO () }
     deriving T.Typeable
@@ -529,7 +633,6 @@ data ManagerSettings = ManagerSettings
       -- ^ Create an insecure connection.
       --
       -- Since 0.1.0
-    -- FIXME in the future, combine managerTlsConnection and managerTlsProxyConnection
     , managerTlsConnection :: IO (Maybe NS.HostAddress -> String -> Int -> IO Connection)
       -- ^ Create a TLS connection. Default behavior: throw an exception that TLS is not supported.
       --
@@ -538,13 +641,13 @@ data ManagerSettings = ManagerSettings
       -- ^ Create a TLS proxy connection. Default behavior: throw an exception that TLS is not supported.
       --
       -- Since 0.2.2
-    , managerResponseTimeout :: Maybe Int
-      -- ^ Default timeout (in microseconds) to be applied to requests which do
-      -- not provide a timeout value.
+    , managerResponseTimeout :: ResponseTimeout
+      -- ^ Default timeout to be applied to requests which do not provide a
+      -- timeout value.
       --
       -- Default is 30 seconds
       --
-      -- Since 0.1.0
+      -- @since 0.5.0
     , managerRetryableException :: SomeException -> Bool
     -- ^ Exceptions for which we should retry our request if we were reusing an
     -- already open connection. In the case of IOExceptions, for example, we
@@ -552,13 +655,13 @@ data ManagerSettings = ManagerSettings
     -- new one.
     --
     -- Since 0.1.0
-    , managerWrapIOException :: forall a. IO a -> IO a
+    , managerWrapException :: forall a. Request -> IO a -> IO a
     -- ^ Action wrapped around all attempted @Request@s, usually used to wrap
     -- up exceptions in library-specific types.
     --
-    -- Default: wrap all @IOException@s in the @InternalIOException@ constructor.
+    -- Default: wrap all @IOException@s in the @InternalException@ constructor.
     --
-    -- Since 0.1.0
+    -- @since 0.5.0
     , managerIdleConnectionCount :: Int
     -- ^ Total number of idle connection to keep open at a given time.
     --
@@ -575,6 +678,12 @@ data ManagerSettings = ManagerSettings
     -- Default: no modification
     --
     -- Since 0.4.4
+    , managerModifyResponse :: Response BodyReader -> IO (Response BodyReader)
+    -- ^ Perform the given modification to a @Response@ after receiving it.
+    --
+    -- Default: no modification
+    --
+    -- @since 0.5.5
     , managerProxyInsecure :: ProxyOverride
     -- ^ How HTTP proxy server settings should be discovered.
     --
@@ -614,16 +723,17 @@ data Manager = Manager
     -- there are no connections to manage.
     , mMaxConns :: Int
     -- ^ This is a per-@ConnKey@ value.
-    , mResponseTimeout :: Maybe Int
+    , mResponseTimeout :: ResponseTimeout
     -- ^ Copied from 'managerResponseTimeout'
     , mRawConnection :: Maybe NS.HostAddress -> String -> Int -> IO Connection
     , mTlsConnection :: Maybe NS.HostAddress -> String -> Int -> IO Connection
     , mTlsProxyConnection :: S.ByteString -> (Connection -> IO ()) -> String -> Maybe NS.HostAddress -> String -> Int -> IO Connection
     , mRetryableException :: SomeException -> Bool
-    , mWrapIOException :: forall a. IO a -> IO a
+    , mWrapException :: forall a. Request -> IO a -> IO a
     , mIdleConnectionCount :: Int
     , mModifyRequest :: Request -> IO Request
     , mSetProxy :: Request -> Request
+    , mModifyResponse      :: Response BodyReader -> IO (Response BodyReader)
     -- ^ See 'managerProxy'
     }
     deriving T.Typeable
